<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2003 Ren Fritz (r.fritz@colorcube.de)
*  All rights reserved
*
*  This script is part of the Typo3 project. The Typo3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * class 'Text statistics' for the 'cc_textfunc' extension.
 *
 * This module analyses English text in either a string or file. Totals are
 * then calculated for the number of characters, words, sentences, blank
 * and non blank (text) lines and paragraphs.
 *
 * Three common readability statistics are also derived, the Fog, Flesch and
 * Kincaid indices.
 *
 * converted from perl to php by Ren Fritz <r.fritz@colorcube.de>
 *
 * -------------------
 * Lingua::EN::Fathom was written by Kim Ryan <kimryan@cpan.org> in 2000. <http://www.data-distillers.com>
 * Copyright (c) 2000-2002 Kim Ryan. All rights reserved. This program is free software; you can redistribute it and/or modify it under the terms of the Perl Artistic License (see http://www.perl.com/perl/misc/Artistic.html).
 * -------------------
 *
 * @author	Ren Fritz <r.fritz@colorcube.de>
 */

# Analyse text stored in a file, reading from the file one line at a time

class tx_cctextfunc_textstat {

	var $num_chars = 0;
	var $num_syllables = 0;
	var $num_words = 0;
	var $num_complex_words = 0;
	var $num_text_lines = 0;
	var $num_blank_lines = 0;
	var $num_paragraphs = 0;
	var $num_sentences = 0;
	var $unique_words = array();

	var $fog = 0;
	var $flesch = 0;
	var $kincaid = 0;

	var $in_paragraph=FALSE;


	function init() {
		$this->num_chars = 0;
		$this->num_syllables = 0;
		$this->num_words = 0;
		$this->num_complex_words = 0;
		$this->num_text_lines = 0;
		$this->num_blank_lines = 0;
		$this->num_paragraphs = 0;
		$this->num_sentences = 0;
		$this->unique_words = array();
		$this->relevant_words = array();
		$this->file_name = '';


		$this->fog = 0;
		$this->flesch = 0;
		$this->kincaid = 0;
	}

	/*
	 * Analyse text stored in a file, reading from the file one line at a time
	 */
	function analyse_file ($file_name) {
		$fd = fopen ($file_name, 'r');
		while (!feof($fd)) {
			$one_line = fgets($fd, 4096);
			$this->_analyse_line($one_line);

		}
		fclose ($fd);
		$this->_calculate_readability();
	}




	/*
	 * Analyse a block of text, stored as a string or an array.
	 * The string may contain line terminators.
	 */
	function analyse_block ($block) {
		if (!is_array($block)) {
			$block = explode(chr(10),$block);
		}
		while (list(,$one_line)=each($block)) {
			$this->_analyse_line($one_line);

		}
		$this->_calculate_readability();
	}


	/*
	 * Increment number of text lines, blank lines and paragraphs
	 */
	function _analyse_line ($one_line){

		if (preg_match ("/\w/", $one_line)) {
			#      chomp($one_line);
			$this->_analyse_words($one_line);
			$this->num_text_lines++;

			if ( !$this->in_paragraph ) {
				$this->num_paragraphs++;
				$this->in_paragraph = TRUE;
			}
		} else { # empty or blank line
			$this->num_blank_lines++;
			$this->in_paragraph = FALSE;
		}
	}


	# Try to detect real words in line. Increment syllable, word, complex word,
	# and sentence counters.

	function _analyse_words($one_line) {

		$this->num_chars += strlen($one_line);

		# Word found, such as: twice, BOTH, a, I'd, non-plussed ..
		# Ignore words like K12, &, X.Y.Z ...
		preg_match_all ("/\b([a-z][-'a-z]*)\b/i", $one_line, $matches);
	#debug($matches);
		#while (list($one_word)=each($matches) )   {
		for ($i=0; $i< count($matches[0]); $i++) {
			$one_word=$matches[1][$i];

			# Try to filter out acronyms and  abbreviations by accepting
			# words with a vowel sound. This won't work for GPO etc.
			//next unless $one_word =~ /[aeiouy]/i;
			if (!preg_match ('/[aeiouy]/i', $one_word)) {
				continue;
			}
			# Test for valid hyphenated word like be-bop
			if (preg_match ('/-/', $one_word)) {
				//next unless $one_word =~ /[a-z]{2,}-[a-z]{2,}/i;
				if (!preg_match ('/[a-z]{2,}-[a-z]{2,}/i', $one_word)) {
					continue;
				}
			}

			# word frequency count
			$this->unique_words[strtolower($one_word)]++;

			$this->num_words++;

			# Use subroutine from Lingua::EN::Syllable
			$num_syllables_current_word = $this->syllable($one_word);
			$this->num_syllables += $num_syllables_current_word;

			# Required for Fog index, count non hyphenated words of 3 or more
			# syllables. Should add check for proper names in here as well
			if ( $num_syllables_current_word > 2 AND !preg_match ('/-/', $one_word) )      {
				$this->num_complex_words++;
			}
		}

		# Remove '.'s to denote common abbreviations . By requiring a following space
		# we know the '.' is not also ending the sentence.
		$abbr = array(
			# People's titles
			"'Mr\. 'i" => 'Mr ',
			"'Mrs\. 'i" => 'Mrs ',
			"'Ms\. 'i" => 'Ms ',
			"'M\/s\. 'i" => "M\/s ",
			"'Dr\. 'i" => 'Dr ',
			"'Prof\. 'i" => 'Prof ',
			"'Det\. 'i" => 'Det ',
			"'Insp\. 'i" => 'Insp ',
			#german
			"'Hr\. 'i" => 'Hr ',
			"'Fr\. 'i" => 'Fr ',
			# Commercial abbreviations
			"'Pty\. 'i" => 'Pty ',
			"'PLC\. 'i" => 'PLC ',
			"'Ltd\. 'i" => 'Ltd ',
			"'Inc\. 'i" => 'Inc ',
			# Other abbreviations
			"'etc\. 'i" => 'etc ',
			"'vs\. 'i" => 'vs ',
		);
		$search = array_keys($abbr);
		$replace = array_values($abbr);
		$one_line = preg_replace ($search, $replace, $one_line);

		# Remove quotation marks as a quote followed by a full stop will not be
		# correctly detected by the following regexps.
		$one_line = preg_replace ("'\"'", '', $one_line);
		$one_line = preg_replace ("'\''", '', $one_line);

		# Search for '.', '?' or '!'  to end a sentence.
		if ( $cnt = preg_match_all("/\b\s*[.!?]\s*\b/", $one_line, $dummy)) {
			$this->num_sentences += $cnt;
		}
		# Check for final sentence, with no following words.
		if (preg_match("/\b\s*[.!?]\s*$/", $one_line)) {
			$this->num_sentences++;
		}
	}

	/*
	 * Determine the three readability indices
	 */
	function _calculate_readability() {
		# $this->_calculate_relevant_words();
		if ($this->num_sentences AND $this->num_words)   {
			$this->words_per_sentence = $this->num_words / $this->num_sentences;
			$this->syllables_per_word = $this->num_syllables / $this->num_words;
			$this->percent_complex_words = ($this->num_complex_words / $this->num_words) * 100;
			$this->fog = ($this->words_per_sentence +  $this->percent_complex_words) * 0.4;
			$this->flesch =  206.835 - (1.015 * $this->words_per_sentence) - (84.6 * $this->syllables_per_word);
			$this->kincaid =  (11.8 * $this->syllables_per_word) + (0.39 * $this->words_per_sentence) - 15.59;
		} else {
			$this->words_per_sentence = 0;
			$this->syllables_per_word = 0;
			$this->fog = 0;
			$this->flesch = 0;
			$this->kincaid = 0;
		}
	}



	function _calculate_relevant_words() {
		reset($this->unique_words);
		while(list($key,$val)=each($this->unique_words)) {
			if (strlen($key)>3 AND $val > 1) {
				$this->relevant_words[$key] = $val;
			}
		}

	}
	// -------------------------------------------------

	# Lingua::EN::Syllable : utility function for counting syllables in words

	# COPYRIGHT
	# Distributed under the same terms as Perl. Contact the author with any questions.

	# AUTHOR
	# Greg Fast (gdf@imsa.edu)

	# note that this is not infallible.  it does fail for some percentage of
	# words (10% seems a good guess)...  so it's useful for approximation, but
	# don't use this for running your nuclear reactor..




	# basic algortithm:
	# each vowel-group indicates a syllable, except for:
	#  final (silent) e
	#  'ia' ind two syl




	# (comments refer to titan's /usr/dict/words)
	# [1] alien, salient, but not lien or ebbullient...
	#     (those are the only 2 exceptions i found, there may be others)
	# [2] exception for 7 words:
	#     coadjutor coagulable coagulate coalesce coalescent coalition coaxial


	function syllable ($word) {

		# @AddSyl and @SubSyl list regexps to massage the basic count.
		# Each match from @AddSyl adds 1 to the basic count, each @SubSyl match -1
		# Keep in mind that when the regexps are checked, any final 'e' will have
		# been removed, and all '\'' will have been removed.

		$SubSyl = array(
			'cial',
			'tia',
			'cius',
			'cious',
			'giu',              # belgium!
			'ion',
			'iou',
			'sia$',
			'.ely$',             # absolutely! (but not ely!)
			);
		$AddSyl = array(
			'ia',
			'riet',
			'dien',
			'iu',
			'io',
			'ii',
			'[aeiouym]bl$',     # -Vble, plus -mble
			'[aeiou]{3}',       # agreeable
			'^mc',
			'ism$',             # -isms
			'([^aeiouy])\1l$',  # middle twiddle battle bottle, etc.
			'[^l]lien',         # alien, salient [1]
			'^coa[dglx].',      # [2]
			'[^gq]ua[^auieo]',  # i think this fixes more than it breaks
			'dnt$',           # couldn't
			);

		$word = strtolower ($word); # $word =~ tr/A-Z/a-z/;
		$word = preg_replace ("'\''", '', $word); # fold contractions.  not very effective.
		$word = preg_replace ("'e$'", '', $word);

		$scrugg = preg_split ('/[^aeiouy]+/', $word); # '-' should perhaps be added?
		$syl = 0;
		# special cases
		foreach ($SubSyl as $val) {
			if (preg_match('/'.$val."/", $word)) {
				$syl--;
			}
		}
		foreach ($AddSyl as $val) {
			if (preg_match('/'.$val."/", $word)) {
				$syl++;
			}
		}
		if (strlen($word)==1) { 	# 'x'
			$syl++;
		}
		# count vowel groupings
		$syl += count($scrugg);
		$syl = ($syl==0) ? $syl=1 : $syl;
		# got no vowels? ('the', 'crwth')

		return $syl;
	}


}



if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/cc_textfunc/class.tx_cctextfunc_textstat.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/cc_textfunc/class.tx_cctextfunc_textstat.php']);
}

 ?>
