<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2004 Ren Fritz ()
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * Module 'Recent changes' for the 'user_recentchanges' extension.
 *
 * @author  Ren Fritz <r.fritz@colorcube.de>
 */



	// DEFAULT initialization of a module [BEGIN]
unset($MCONF);
require ('conf.php');
require ($BACK_PATH.'init.php');
require ($BACK_PATH.'template.php');

	// Include locallang file for module
$LANG->includeLLFile('EXT:user_recentchanges/mod1/locallang.php');

	// Include module base class
require_once (PATH_t3lib.'class.t3lib_scbase.php');

	// This checks permissions and exits if the users has no permission for entry.
$BE_USER->modAccess($MCONF,1);

	// DEFAULT initialization of a module [END]


class user_recentchanges_module1 extends t3lib_SCbase {


	/**
	 * Adds items to the ->MOD_MENU array. Used for the function menu selector.
	 *
	 * @return  void
	 */
	function menuConfig()   {
		global $LANG;
		$this->MOD_MENU = Array (
			'function' => Array (
				'newAndUpdated' => $LANG->getLL('newAndUpdated'),
				'updated' => $LANG->getLL('updated'),
				'new' => $LANG->getLL('new'),
			),
			'showRootline' => 1,
		);
		parent::menuConfig();
	}


	/**
	 * Main function of the module. Write the content to $this->content
	 *
	 * @return  void
	 */
	function main() {
		global $BE_USER,$LANG,$BACK_PATH;


		if ($BE_USER->user['admin'])    {

				// Init the module doc
			$this->doc = t3lib_div::makeInstance('bigDoc');
			$this->doc->backPath = $BACK_PATH;
			$this->doc->form = '<form action="" method="POST">';

				// JavaScript, used for menus
			$this->doc->JScode = $this->doc->wrapScriptTags('
					function jumpToUrl(URL) {
						document.location = URL;
					}
			');

				// Ouput page header
			$this->content.= $this->doc->startPage($LANG->getLL('title'));
			$this->content.= $this->doc->header($LANG->getLL('title'));
			$this->content.= $this->doc->spacer(5);

				// Output menu
			$menu = array();
			$menu[] = t3lib_BEfunc::getFuncMenu($this->id, 'SET[function]', $this->MOD_SETTINGS['function'], $this->MOD_MENU['function']);
			$menu[] = t3lib_BEfunc::getFuncCheck($this->id, 'SET[showRootline]', $this->MOD_SETTINGS['showRootline']).' '.$LANG->getLL('showRootline');
			$this->content.= $this->doc->section('', implode('<br />',$menu));
			$this->content.= $this->doc->divider(5);

				// Render content:
			$this->moduleContent();

				// ShortCut
			if ($BE_USER->mayMakeShortcut())    {
				$this->content.= $this->doc->spacer(20).$this->doc->section('', $this->doc->makeShortcutIcon('id',implode(',',array_keys($this->MOD_MENU)), $this->MCONF['name']));
			}

			$this->content.= $this->doc->spacer(10);

		} else {
				// If no access: output message

			$this->doc = t3lib_div::makeInstance('bigDoc');
			$this->doc->backPath = $BACK_PATH;

			$this->content.= $this->doc->startPage($LANG->getLL('title'));
			$this->content.= $this->doc->header($LANG->getLL('title'));
			$this->content.= $this->doc->spacer(5);
			$this->content.= $this->doc->section('', $LANG->getLL('admins_only'));
			$this->content.= $this->doc->spacer(10);
		}
	}


	/**
	 * Prints out the module HTML
	 *
	 * @return  void
	 */
	function printContent() {
		$this->content.= $this->doc->endPage();
		echo $this->content;
	}


	/**
	 * Generates the module content
	 *
	 * @return  void
	 */
	function moduleContent()    {
		global $LANG;

		$mode = $this->MOD_SETTINGS['function'];
		$content = $this->getRecentChangesTable($mode);
		$this->content.= $this->doc->section($LANG->getLL($mode.'_header'),$content,0,1);
	}


	/**
	 * Render the table with recently changed records
	 *
	 * @param   string      Mode from $this->MOD_SETTINGS['function']
	 * @return  string      Rendered Table
	 */
	function getRecentChangesTable($mode)   {
		global $BACK_PATH, $BE_USER, $LANG, $TCA;

			// Set sys_log actions depending on selected mode
		if ($mode=='new') {
			$action = '1';
		} elseif ($mode=='updated') {
			$action = '2';
		} else {
			$action = '1,2';
		}

			// Query sys_log for non-deleted pages only
		$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery(
						'sys_log.*',
						'sys_log,pages',
						'pages.uid=sys_log.event_pid'.
							' AND sys_log.event_pid>0 AND sys_log.type=1 AND sys_log.action IN ('.$action.') AND sys_log.error=0'.
							t3lib_BEfunc::deleteClause('pages'),
						'',
						'tstamp DESC',
						40
					);


			// init table layout
		$tableLayout = array (
			'table' => array ('<table border="0" cellpadding="1" cellspacing="1" class="typo3-recent-edited">', '</table>'),
			'defRow' => array (
				'tr' => array('<tr class="bgColor4">','</tr>'),
				'defCol' => Array('<td valign="top">','</td>')
			)
		);

		$table = array();
		$tr = 0;

		while($logRow = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res))    {

			$page_id = $logRow['event_pid'];
			$pageRow = t3lib_BEfunc::getRecord('pages', $page_id);

			if (is_array($pageRow)) {

					// Create output item for pages record
				$contentPageLink = $this->getItemFromRecord('pages', $pageRow);

					// Create output text describing the age
				$contentAge = t3lib_BEfunc::dateTimeAge($logRow['tstamp'], 1);


				$contentElementLink = '';
				$contentUser = '';

					// Fetch record if table is not "pages"
				if(!($logRow['tablename']=='pages'))    {
					$elementRow = t3lib_BEfunc::getRecord($logRow['tablename'], $logRow['recuid']);

						// If record is deleted continue with next log entry
					if (!is_array($elementRow)) {
						continue;
					}

						// Create output item for non pages record
					if(!($logRow['tablename']=='pages'))    {
						$contentElementLink = $this->getItemFromRecord($logRow['tablename'], $elementRow);
					}
				}

					// Create user item
				$userRow = t3lib_BEfunc::getRecord('be_users', $logRow['userid']);
				if(is_array($userRow)) {
					$contentUser = $this->wrapEditLink(htmlspecialchars($userRow['username'].' ('.$userRow['uid'].')'), 'be_users', $userRow['uid']);

						// Use different row color if record was edited by current user
					if ($userRow['uid']==$BE_USER->user['uid']) {
						$tableLayout[$tr]['tr'] = array('<tr class="bgColor5">','</tr>');
					}
				}

					// Add row to table
				$td=0;
				$table[$tr][$td++] = $contentPageLink;
				$table[$tr][$td++] = $contentElementLink;
				$table[$tr][$td++] = $contentUser;
				$table[$tr][$td++] = $contentAge;
				$tr++;
			}
		}

			// Return rendered table
		return $this->doc->table($table, $tableLayout);
	}


	/**
	 * Returns a linked icon with title from a record
	 *
	 * @param   string      Table name (tt_content,...)
	 * @param   array       Record array
	 * @return  string      Rendered icon
	 */
	function getItemFromRecord($table, $row) {
		global $BACK_PATH, $LANG, $TCA, $BE_USER;

		$iconAltText = t3lib_BEfunc::getRecordIconAltText($row, $table);

			// Prepend table description for non-pages tables
		if(!($table=='pages')) {
			$iconAltText = $LANG->sl($TCA[$table]['ctrl']['title']).': '.$iconAltText;
		}

			// Create record title or rootline for pages if option is selected
		if($table=='pages' AND $this->MOD_SETTINGS['showRootline']) {
			$elementTitle = t3lib_BEfunc::getRecordPath($row['uid'], '1=1', 0);
			$elementTitle = t3lib_div::fixed_lgd_cs($elementTitle, -($BE_USER->uc['titleLen']));
		} else {
			$elementTitle = t3lib_BEfunc::getRecordTitle($table, $row, 1);
		}

			// Create icon for record
		$elementIcon = t3lib_iconworks::getIconImage($table, $row, $BACK_PATH, 'class="c-recicon" title="'.$iconAltText.'"');

			// Return item with edit link
		return $this->wrapEditLink($elementIcon.$elementTitle, $table, $row['uid']);
	}


	/**
	 * Wraps an edit link around a string.
	 * Creates a page module link for pages, edit link for other tables.
	 *
	 * @param   string      The string to be wrapped
	 * @param   string      Table name (tt_content,...)
	 * @param   integer     uid of the record
	 * @return  string      Rendered link
	 */
	function wrapEditLink($str, $table, $id)    {
		global $BACK_PATH, $LANG, $BE_USER;

		if($table=='pages') {
			$editOnClick = "top.fsMod.recentIds['web']=".$id.";top.goToModule('web_layout',1);";
		} else {
			$params = '&edit['.$table.']['.$id.']=edit';
			$editOnClick = t3lib_BEfunc::editOnClick($params, $BACK_PATH);
		}
		return '<a href="#" onclick="'.htmlspecialchars($editOnClick).'">'.$str.'</a>'; ;
	}
}



if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/user_recentchanges/mod1/index.php'])   {
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/user_recentchanges/mod1/index.php']);
}




// Make instance:
$SOBE = t3lib_div::makeInstance('user_recentchanges_module1');
$SOBE->init();

// Include files?
foreach($SOBE->include_once as $INC_FILE)   include_once($INC_FILE);

$SOBE->main();
$SOBE->printContent();

?>