<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2004 Ren Fritz (r.fritz@colorcube.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * Plugin 'Countdown TypoTag' for the 'user_tag_countdown' extension.
 *
 * @author  Ren Fritz <r.fritz@colorcube.de>
 */


require_once(PATH_tslib.'class.tslib_pibase.php');

class user_tagcountdown_pi1 extends tslib_pibase {
	var $prefixId = 'user_tagcountdown_pi1';        // Same as class name
	var $scriptRelPath = 'pi1/class.user_tagcountdown_pi1.php'; // Path to this script relative to the extension dir.
	var $extKey = 'user_tag_countdown'; // The extension key.


	/**
	 * processes the <countdown> tag
	 */
	function main($content,$conf)   {
			// count items for JavaScript usage
		$GLOBALS['T3_VAR']['ext'][$this->prefixId]['count']++;

			// get parameter from tag
		$unit = $this->cObj->parameters['unit'];
			// get parameter from TypoScript
		$unit = $unit ? $unit : $this->cObj->stdWrap($conf['unit'], $conf['unit.']);


		if ($unit=='sec')   {
			$divider = 1;
		} elseif ($unit=='min') {
			$divider = 60;
		} elseif ($unit=='hour')    {
			$divider = 60 * 60;
		} else {
				// day - default
			$divider = 60 * 60 * 24;
		}

		$date = $this->cObj->getCurrentVal();
		$timestamp = strtotime($date);
		$delta = $timestamp - time();
		$days = intval($delta / $divider) + 1;


			// check if "animate" parameter is set in tag
		$animate = isset($this->cObj->parameters['animate']);
			// get parameter from TypoScript
		$animate = $animate ? $animate : $this->cObj->stdWrap($conf['animate'], $conf['animate.']);

			// do animation for seconds or minutes only
		if ($animate AND ($unit=='sec' OR $unit=='min')) {
				// unique id for every element (HTML DOM doesn't accept underscores)
			$domId = str_replace('_','-',$this->prefixId).'-'.$GLOBALS[$this->prefixId]['count'];
				// include JS code
			$this->addJsCounter($domId, $days, $unit);
				// add an id to the content
			$days = '<span id="'.$domId.'">'.$days.'</span>';
		}

		return $days;
	}




	/**
	 * Add JavaScript counter code to the page
	 *
	 * @param   string      dom id
	 * @param   integer     start value for the counter
	 * @param   string      unit: 'sec' or 'min'
	 * @return  void
	*/
	function addJsCounter($id, $countdown, $unit)   {
			// include JS code once
		if (!$GLOBALS['TSFE']->additionalHeaderData[$this->prefixId]) {

			$jsCode = <<<EOD
				{$this->prefixId}ObjArr = new Array();

				function {$this->prefixId}(id,countdown,unit) {
					this.id = id;
					this.countdown = countdown;
					setInterval("{$this->prefixId}ObjArr['"+id+"'].showcount()",unit);
				}

				{$this->prefixId}.prototype.showcount = function () {
						this.countdown = this.countdown-1;
						element = document.getElementById(this.id);
						element.innerHTML = this.countdown;
				}
EOD;
				// wrap JavaScript in script tags and add to page header
			$GLOBALS['TSFE']->additionalHeaderData[$this->prefixId] = t3lib_div::wrapJS($jsCode);
		}

		$unit = ($unit=='min') ? 60000 : 1000;

			// add JS onload handler
		$GLOBALS['TSFE']->JSeventFuncCalls['onload'][$id] = "{$this->prefixId}ObjArr['{$id}'] = new {$this->prefixId}('{$id}', {$countdown}, '{$unit}');";
	}
}



if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/user_tag_countdown/pi1/class.user_tagcountdown_pi1.php'])  {
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/user_tag_countdown/pi1/class.user_tagcountdown_pi1.php']);
}

?>